package com.unissoft.config;

import com.unissoft.exception.MyAccessDeniedHandler;
import com.unissoft.exception.MyAuthExceptionEntryPoint;
import com.unissoft.service.impl.OauthClientDetailsServicel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.security.config.annotation.ObjectPostProcessor;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableResourceServer;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configurers.ResourceServerSecurityConfigurer;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.security.oauth2.provider.token.store.JdbcTokenStore;
import org.springframework.security.oauth2.provider.token.store.JwtAccessTokenConverter;
import org.springframework.security.oauth2.provider.token.store.redis.RedisTokenStore;
import org.springframework.security.web.access.intercept.FilterSecurityInterceptor;

import javax.sql.DataSource;

/**
 * 配置资源服务器适配器
 */
@Configuration
@EnableResourceServer //@EnableResourceServer来开启/关闭过滤器
@EnableGlobalMethodSecurity(prePostEnabled = true) //来判断用户对某个控制层的方法是否具有访问权限
public class ResourceSeverConfig extends ResourceServerConfigurerAdapter {

    @Autowired
    private MyConfig myConfig;
    @Autowired
    private OauthClientDetailsServicel oauthClientDetailsService;
    @Autowired
    private DataSource dataSource;
    @Autowired
    private RedisConnectionFactory redisConnectionFactory;
    @Autowired
    private MySecurityMetadataSource mySecurityMetadataSource;
    @Autowired
    private MyAccessDecisionManager myAccessDecisionManager;

    /**
     * 对Jwt签名时，增加一个密钥
     * JwtAccessTokenConverter：对Jwt来进行编码以及解码的类
     * 这个bean要与wsm-oauth服务AuthorizationServerConfig类JwtAccessTokenConverter方法相对应
     */
    @Bean
    public JwtAccessTokenConverter jwtAccessTokenConverter() {

        JwtAccessTokenConverter jwtAccessTokenConverter = new JwtAccessTokenConverter();

        //对称加密方式
        jwtAccessTokenConverter.setSigningKey("jwt_wsm");

        return jwtAccessTokenConverter;
    }

    /**
     * application.yml有解释
     *
     * @return
     */
    @Bean
    public TokenStore tokenStore() {
        // 存入数据库
        return new JdbcTokenStore(dataSource);
        //  存入redis
//        return new RedisTokenStore(redisConnectionFactory);
    }

    /**
     * 根据client_id设置资源id
     * 设置自定义授权异常信息
     */
    @Override
    public void configure(ResourceServerSecurityConfigurer resources) {
        String resourceIds = oauthClientDetailsService.getById(myConfig.getClientId()).getResourceIds();
//        String resourceIds = oauthClientDetailsService.getResourceIdsByClientId(myConfig.getClientId());
        //设置客户端所能访问的资源id集合(默认取第一个是本服务的资源)
        resources.resourceId(resourceIds.split(",")[0]).stateless(true);
        resources.tokenStore(tokenStore());
        //自定义Token异常信息,用于token校验失败返回信息
        resources.authenticationEntryPoint(new MyAuthExceptionEntryPoint())
                //授权异常处理
                .accessDeniedHandler(new MyAccessDeniedHandler());
    }

    /**
     * 配置资源服务，根据项目code从数据库读取出来的角色，资源url信息（即接口地址）给接口使用security权限。
     * 如哪些不需要授权能访问，哪些需要登录授权后能访问，哪些需要用户拥有这些角色才能访问。
     */
    @Override
    public void configure(HttpSecurity http) throws Exception {
        http.csrf().disable();//防csrf攻击 禁用
        http.authorizeRequests().antMatchers("/public/**").permitAll();//可以访问

        //启用权限
        if (myConfig.getIsUseSecurity()) {
            http.authorizeRequests().antMatchers("/test/**").authenticated();

            //权限配置1（第一种实现：动态、修改了权限不需要重启项目，只需调用mySecurityMetadataSource类loadResourceDefine方法刷新权限，重新登陆获取token即可生效）
            mySecurityMetadataSource.loadResourceDefine();//加载权限表中所有操作请求权限
            http.authorizeRequests().withObjectPostProcessor(new ObjectPostProcessor<FilterSecurityInterceptor>() {
                @Override
                public <O extends FilterSecurityInterceptor> O postProcess(O o) {
                    o.setSecurityMetadataSource(mySecurityMetadataSource); //动态获取url权限配置
                    o.setAccessDecisionManager(myAccessDecisionManager); //权限判断
                    return o;
                }
            });
        }
    }
}
